# Copyright 2022 Animatea
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Adapters of aiotense.application.ports."""
__all__ = ["TimedeltaConverter"]

from datetime import timedelta
from typing import Any

from aiotense.application.ports import converters


class TimedeltaConverter(converters.AbstractConverter[timedelta]):
    async def convert(self, value: Any) -> timedelta:
        if not isinstance(value, (int, float)):
            raise ValueError("Value must be instance of (int, float).")

        return timedelta(seconds=float(value))
